## $Id: tests.py 10529 2013-08-23 10:38:21Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import os
from zope.component import getUtility
from zope.catalog.interfaces import ICatalog
from hurry.workflow.interfaces import IWorkflowState
from waeup.kofa.students.tests.test_browser import StudentsFullSetup
from waeup.kofa.applicants.tests.test_browser import ApplicantsFullSetup
from waeup.kofa.configuration import SessionConfiguration
from waeup.fceokene.testing import FunctionalLayer

# Also run tests that send requests to external servers?
#   If you enable this, please make sure the external services
#   do exist really and are not bothered by being spammed by a test programme.
EXTERNAL_TESTS = False

def external_test(func):
    if not EXTERNAL_TESTS:
        myself = __file__
        if myself.endswith('.pyc'):
            myself = myself[:-2]
        print "WARNING: external tests are skipped!"
        print "WARNING: edit %s to enable them." % myself
        return
    return func


class InterswitchTestsStudents(StudentsFullSetup):
    """Tests for the Interswitch payment gateway.
    """

    layer = FunctionalLayer

    def setUp(self):
        super(InterswitchTestsStudents, self).setUp()
        self.certificate.study_mode = 'nce_ft'
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('cleared')
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['schoolfee']
        self.browser.getControl("Create ticket").click()
        self.assertMatches('...ticket created...',
                           self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertMatches('...Amount Authorized...',
                           self.browser.contents)
        self.assertTrue(
            '<span>12645.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url


#    def callback_url(self, payment_url, resp, apprAmt):
#        return payment_url + (
#            '/isw_callback?echo=' +
#            '&resp=%s' +
#            '&desc=Something went wrong' +
#            '&txnRef=p1331792385335' +
#            '&payRef=' + '&retRef=' +
#            '&cardNum=0' +
#            '&apprAmt=%s' +
#            '&url=http://xyz') % (resp, apprAmt)

    def test_interswitch_form(self):

        self.assertEqual(self.student['payments'][self.value].provider_amt, 0.0)
        self.assertEqual(self.student['payments'][self.value].gateway_amt, 0.0)
        self.assertEqual(self.student['payments'][self.value].thirdparty_amt, 0.0)
        # Manager can access InterswitchForm for the created school fee ticket
        self.browser.getLink("CollegePAY", index=0).click()
        # Split amounts have been set
        self.assertEqual(self.student['payments'][self.value].provider_amt, 1600.0)
        self.assertEqual(self.student['payments'][self.value].gateway_amt, 150.0)
        self.assertEqual(self.student['payments'][self.value].thirdparty_amt, 1400.0)
        self.assertMatches('...Total Amount Authorized:...',
                           self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="1264500.0" />'
            in self.browser.contents)
        self.assertTrue(
            'item_name="School Fee" item_amt="949500" bank_id="117" acct_num="1012044015"'
            in self.browser.contents)
        self.assertTrue(
            'item_name="FCEOkene Split" item_amt="140000" bank_id="117" acct_num="1012044039"'
            in self.browser.contents)
        self.assertTrue(
            'item_name="BT Education" item_amt="160000" bank_id="31" acct_num="0026781725"'
            in self.browser.contents)

        # Maintenance fee payment

        self.browser.open(self.payments_path)
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(
            name="form.p_category").value = ['hostel_maintenance']
        self.browser.getControl("Create ticket").click()
        self.assertMatches('...You have not yet booked accommodation...',
                           self.browser.contents)
        # Students have to book bed first
        self.browser.open(self.acco_path)
        IWorkflowState(self.student).setState('admitted')
        self.browser.getLink("Book accommodation").click()
        self.assertFalse('Activation Code:' in self.browser.contents)
        self.browser.getControl("Create bed ticket").click()
        # Bed is randomly selected but, since there is only
        # one bed for this student, we know that ...
        self.assertFalse('Hall 1, Block A, Room 101, Bed A'
                           in self.browser.contents)
        self.assertTrue('(see payment slip)'
                           in self.browser.contents)
        self.assertTrue('ticket created'
                           in self.browser.contents)
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(
            name="form.p_category").value = ['hostel_maintenance']
        self.browser.getControl("Create ticket").click()
        self.assertMatches('...ticket created...',
                           self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        value = ctrl.options[1]
        self.browser.getLink(value).click()
        self.assertMatches('...Amount Authorized...',
                           self.browser.contents)
        self.assertTrue(
            '<span>4150.0</span>' in self.browser.contents)
        # p_item is not unveiled
        self.assertFalse('Hall 1, Block A, Room 101, Bed A'
            in self.browser.contents)
        self.assertTrue('(visible after successful payment)'
            in self.browser.contents)
        self.payment_url = self.browser.url
        self.assertEqual(self.student['payments'][value].provider_amt, 0.0)
        self.assertEqual(self.student['payments'][value].gateway_amt, 0.0)
        self.assertEqual(self.student['payments'][value].thirdparty_amt, 0.0)
        # Manager can access InterswitchForm
        self.browser.getLink("CollegePAY", index=0).click()
        # Split amounts have been set
        self.assertEqual(self.student['payments'][value].provider_amt, 0.0)
        self.assertEqual(self.student['payments'][value].gateway_amt, 150.0)
        self.assertEqual(self.student['payments'][value].thirdparty_amt, 0.0)
        # The total amount to be processed by Interswitch
        # has been reduced by the Interswitch fee of 150 Nairas
        self.assertMatches('...Total Amount Authorized:...',
                           self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="415000.0" />'
            in self.browser.contents)
        self.assertTrue(
            'item_name="Hostel Maintenance Fee" item_amt="400000" bank_id="117" acct_num="1012044132"'
            in self.browser.contents)
        # BT does nor charge a fee for maintenance fee
        self.assertFalse("BT Education" in self.browser.contents)
        # p_item is not unveiled
        self.assertFalse('Hall 1, Block A, Room 101, Bed A'
            in self.browser.contents)
        self.assertTrue('(visible after successful payment)'
            in self.browser.contents)
        # If the ticket is paid coordinates are shown
        self.student['payments'][value].p_state = 'paid'
        self.browser.open(self.payment_url)
        self.assertTrue('Hall 1, Block A, Room 101, Bed A'
            in self.browser.contents)
        self.assertFalse('(visible after successful payment)'
            in self.browser.contents)

        # Acceptance fee payment

        self.browser.open(self.payments_path)
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(
            name="form.p_category").value = ['clearance']
        self.browser.getControl("Create ticket").click()
        self.assertMatches('...ticket created...',
                           self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        value = ctrl.options[2]
        self.browser.getLink(value).click()
        self.assertMatches('...Amount Authorized...',
                           self.browser.contents)
        self.assertTrue(
            '<span>3606.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url
        self.assertEqual(self.student['payments'][value].provider_amt, 0.0)
        self.assertEqual(self.student['payments'][value].gateway_amt, 0.0)
        self.assertEqual(self.student['payments'][value].thirdparty_amt, 0.0)
        # Manager can access InterswitchForm
        self.browser.getLink("CollegePAY", index=0).click()
        # Split amounts have been set
        self.assertEqual(self.student['payments'][value].provider_amt, 0.0)
        self.assertEqual(self.student['payments'][value].gateway_amt, 150.0)
        self.assertEqual(self.student['payments'][value].thirdparty_amt, 0.0)
        # The total amount to be processed by Interswitch
        # has been reduced by the Interswitch fee of 150 Nairas
        self.assertMatches('...Total Amount Authorized:...',
                           self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="360600.0" />' in
            self.browser.contents)
        self.assertTrue(
            'item_name="Acceptance Fee" item_amt="345600" bank_id="8" acct_num="2003670143"'
            in self.browser.contents)
        # BT does nor charge a fee for maintenance fee
        self.assertFalse("BT Education" in self.browser.contents)

#    @external_test
#    def test_callback(self):

        # Manager can call callback manually
#        self.browser.open(self.callback_url(self.payment_url, 'XX', '300'))
#        self.assertMatches('...Unsuccessful callback: Something went wrong...',
#                          self.browser.contents)
#        self.assertMatches('...Failed...',
#                           self.browser.contents)
#        self.browser.open(self.payment_url + '/isw_callback')
#        self.assertMatches('...Unsuccessful callback: Incomplete query string...',
#                          self.browser.contents)
#        self.assertMatches('...Failed...',
#                           self.browser.contents)
#        self.browser.open(self.callback_url(self.payment_url, '00', '300000'))
#        self.assertMatches('...Wrong amount...',
#                          self.browser.contents)
#        self.browser.open(self.callback_url(self.payment_url, '00', '4000000'))
#        self.assertMatches('...Valid callback received...',
#                          self.browser.contents)

    @external_test
    def test_webservice(self):
        # First we have open InterswitchPageStudent to set provider_amt
        # and gateway_amt
        self.browser.open(self.payment_url + '/goto_interswitch')
        # Now we can call the webservice
        self.browser.open(self.payment_url + '/request_webservice')
        #self.assertMatches('...Unsuccessful callback...',
        #                  self.browser.contents)
        self.assertMatches('...Unsuccessful callback...',
                          self.browser.contents)
        # The payment is now in state failed ...
        self.assertMatches('...<span>Failed</span>...',
                          self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('failed', 'failed')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'failed')

        # Let's replace the p_id with a valid p_id of the FCEOkene
        # live system. This is definitely not an appropriate
        # solution for testing, but we have no choice since
        # Interswitch doesn't provide any interface
        # for testing.
        payment = self.student['payments'][self.value]
        payment.p_id = 'p3536651296379'
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Callback amount does not match...',
                          self.browser.contents)
        # The payment is now in state failed ...
        self.assertMatches('...<span>Failed</span>...',
                          self.browser.contents)
        # Let's replace the amount autorized with the amount of the
        # live system payment
        payment.amount_auth = payment.r_amount_approved
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Successful payment...',
                          self.browser.contents)
        # The payment is now in state paid ...
        self.assertMatches('...<span>Paid</span>...',
                          self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('paid', 'paid')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'paid')
        # Approval is logged in students.log ...
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'students.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'zope.mgr - '
            'waeup.fceokene.interswitch.browser.InterswitchPaymentRequestWebservicePageStudent - '
            'K1000000 - successful schoolfee payment: p3536651296379\n'
            in logcontent)
        # ... and in payments.log
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'payments.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            '"zope.mgr",K1000000,p3536651296379,schoolfee,'
            '3150.0,00,1600.0,150.0,1400.0,,,\n'
            in logcontent)

class InterswitchTestsApplicants(ApplicantsFullSetup):
    """Tests for the Interswitch payment gateway.
    """

    layer = FunctionalLayer

    def setUp(self):
        super(InterswitchTestsApplicants, self).setUp()
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.manage_path)
        #IWorkflowState(self.student).setState('started')
        super(InterswitchTestsApplicants, self).fill_correct_values()
        self.applicantscontainer.application_fee = 1000.0
        self.browser.getControl(name="form.nationality").value = ['NG']
        self.browser.getControl(name="transition").value = ['start']
        self.browser.getControl("Save").click()
        self.browser.getControl("Add online").click()
        self.assertMatches('...ticket created...',
                           self.browser.contents)
        self.assertMatches('...Amount Authorized...',
                           self.browser.contents)
        self.assertMatches(
            '...<span>1000.0</span>...',
            self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.open(self.manage_path)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]


    def test_interswitch_form(self):

        # Manager can access InterswitchForm
        self.browser.open(self.payment_url)
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertMatches('...Total Amount Authorized:...',
                           self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="100000.0" />'
            in self.browser.contents)
        self.assertTrue(
            '<item_detail item_id="1" item_name="application"'
            ' item_amt="35000" bank_id="117" acct_num="1012445289" />'
            in self.browser.contents)


    @external_test
    def test_webservice(self):
        # First we have open InterswitchPageStudent to set provider_amt
        # and gateway_amt
        self.browser.open(self.payment_url + '/goto_interswitch')
        # Now we can call the webservice
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Unsuccessful callback...',
                          self.browser.contents)
        # The payment is now in state failed ...
        self.assertMatches('...<span>Failed</span>...',
                          self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('failed', 'failed')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'failed')

        # Let's replace the p_id with a valid p_id of the Uniben
        # live system. This is definitely not an appropriate
        # solution for testing, but we have no choice since
        # Interswitch doesn't provide any interface
        # for testing.
        payment = self.applicant[self.value]
        payment.p_id = 'p3536651296379'
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Callback amount does not match...',
                          self.browser.contents)
        # The payment is now in state failed ...
        self.assertMatches('...<span>Failed</span>...',
                          self.browser.contents)
        # Let's replace the amount autorized with the amount of the
        # live system payment
        payment.amount_auth = payment.r_amount_approved
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Successful payment...',
                          self.browser.contents)
        # The payment is now in state paid ...
        self.assertMatches('...<span>Paid</span>...',
                          self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('paid', 'paid')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'paid')
        # Approval is logged in students.log ...
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'applicants.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'zope.mgr - '
            'waeup.fceokene.interswitch.browser.InterswitchPaymentRequestWebservicePageApplicant - '
            '%s - successful payment: p3536651296379\n'
            % self.applicant.applicant_id in logcontent)
        # ... and in payments.log
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'payments.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            '"zope.mgr",%s,p3536651296379,application,'
            '3150.0,00,500.0,150.0,0.0,,,\n' % self.applicant.applicant_id
            in logcontent)
